/*
 * Initialization and main function.
 *
 * $LastChangedRevision: 490 $
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include <stdint.h>
#include "CommonFcts.h"
#include "TableStruct.h"
#include "LNet.h"
#include "Services.h"
#include "BlockServicesX2C.h"
#include "CommonFcts.h"
#include "Hardware.h"
#include "X2C.h"
#include "Main.h"

/* generated X2C parameter identifier & block function table */
extern const tParameterTable parameterIdTable[];
extern const tBlockFunctions blockFunctionTable[];

/* X2C Outports */
extern TOutports Outports;
extern TInports Inports;

/* X2C maintenance structure */
volatile tTableStruct TheTableStruct;
volatile tTableStruct* TableStruct = &TheTableStruct;

/* LNet protocol buffersize & node identifier */
#define LNET_BUFFERSIZE ((uint8)255)
#define LNET_NODEID ((uint8)1)

static tSerial interface;
static tLNet protocol;
static uint8 bufferLNet[LNET_BUFFERSIZE];

/* used by init version info */
const struct {
	uint8 date[11];
	uint8 time[8];
} compilationDate = { __DATE__, __TIME__ };

/* private prototypes */
static void initVersionInfo(volatile tTableStruct* tblStruct, uint16 appVersion);


/*
 * Main function.
 */
void main(void)
{
    initTableStruct();
    initSerialGeneric(&interface);
    initSerial(&interface);
    initLNet(&protocol, bufferLNet, LNET_BUFFERSIZE, LNET_NODEID);
    initServiceTable((tProtocol*)&protocol);
    addCoreServices((tProtocol*)&protocol);
    addBlockServices((tProtocol*)&protocol);
    addTableStructProtocol((tProtocol*)&protocol);
    linkSerial((tProtocol*)&protocol, &interface);

	initVersionInfo(TableStruct, (uint16)0x0001);
	TableStruct->DSPState = PRG_LOADED_STATE;

    TableStruct->TFncTable = blockFunctionTable;
    TableStruct->TParamTable = parameterIdTable;

	initHardware();
	KICK_DOG;

    X2C_Init();
    KICK_DOG;

    LED_BLUE_OFF;
    LED_GREEN_OFF;
    LED_RED_OFF;

    while(1)
	{
		/* communication procedure */
		protocol.pCommunicate((tProtocol*)&protocol);
	}
}


/*
 * Main control task.
 */
void mainTask(void)
{
    static int16 cnt = 0;

    /* assign inports */
    Inports.Cnt = cnt++;

    /* update X2C */
    X2C_Update();

    /* update outports */
    if (*Outports.pLED_red > 0) {
        LED_RED_ON;
    } else {
        LED_RED_OFF;
    }

    if (*Outports.pLED_green > 0) {
        LED_GREEN_ON;
    } else {
        LED_GREEN_OFF;
    }

    if (*Outports.pLED_blue > 0) {
        LED_BLUE_ON;
    } else {
        LED_BLUE_OFF;
    }

    /* check buttons */
    if((ROM_GPIOPinRead(GPIO_PORTF_BASE, SWITCH1)&SWITCH1) == 0){	/* SW1 pressed */
    	Inports.Switch1 = INT16_MAX;
    }else{
    	Inports.Switch1 = 0;
    }

    if((ROM_GPIOPinRead(GPIO_PORTF_BASE, SWITCH2)&SWITCH2) == 0){	/* SW2 pressed */
    	Inports.Switch2 = INT16_MAX;
    }else{
    	Inports.Switch2 = 0;
    }

    /* reset watchdog timer */
    KICK_DOG;
}

/*
 * Adds frame compilation date, time and version info to X2C maintenance structure.
 */
static void initVersionInfo(volatile tTableStruct* tblStruct, uint16 appVersion)
{
	tblStruct->framePrgVersion = appVersion;
	tblStruct->framePrgCompDateTime = (uint8*)&compilationDate;
}
